/* -*- mode: c++; tab-width: 4; c-basic-offset: 4 -*- */
/*
 * Copyright (C) 2005 Atmark <atmarkat _AT_ msn _DOT_ com>
 *                    AGAWA Koji <i _AT_ atty _DOT_ jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef MPLAYERCONFIG_H_INCLUDED
#define MPLAYERCONFIG_H_INCLUDED

#include <qcheckbox.h>
#include <qspinbox.h>
#include <qlineedit.h>

#include "configuration.h"
#include "configview.h"
#include "playbackbackend.h"

#ifndef QT4
class QButtonGroup;
#endif

class MPlayerConfiguration
{
public:
    MPlayerConfiguration();

    void write(Config &cf) const;
    void read(Config &cf);

public:
    QString mplayerPath;		//!< MPlayer実行ファイルのパス
    QString mplayerBinLocation();

    const QString mplayerArguments(QString fileExtension);
    void setFormatSpecificConfigDefaults(Configuration &config);

    bool isOverlayEnabled;		//!< オーバレイを使うか
    bool isFullScreenEnabled;	//!< ビデオをフルスクリーン再生するか
    bool isDoubleBufferingEnabled; //!< ダブルバッファリングを行なうか

    enum FramedropMethod {
        FRAMEDROP_NONE = 0,		//!< フレームドロップ無し
        FRAMEDROP_NORMAL = 1,	//!< 通常のフレームドロップ
        FRAMEDROP_HARD = 2		//!< 強力なフレームドロップ
    };
    FramedropMethod framedropMethod; //!< フレームドロップの方法

    bool isCacheEnabled;
    int cacheSize;
    bool isPrefetchFileEnabled;

    bool isAdditionalOptionsEnabled; //!< コマンドラインオプションを追加するか
    QString additionalOptions;	//!< 追加のコマンドラインオプション

#ifdef QT4
    QHash<QString, QString> formatMPlayerArguments;
#else
    QMap<QString, QString> formatMPlayerArguments;
#endif

    // ----------------
    //! ZPlayerを動作させている機種の分類
    enum MachineCategory {
        MACHINE_SLC700,			//!< SL-C700系の機種
        MACHINE_SLC3000,		//!< SL-C3000系の機種
        MACHINE_GENERIC			//!< その他の機種
    };
    //! ZPlayerを動作させている機種の分類を返します。
    MachineCategory machineCategory() const { return machineCategory_; }
    //! ZPlayerを動作させている機種の名前を返します。
    QString machineName() const { return machineName_; }

private:
    void detectMachine();
    MachineCategory machineCategory_;
    QString machineName_;
};

class MPlayerConfig : public ConfigView
{
    Q_OBJECT
public:
    MPlayerConfig(Configuration& configuration, QWidget *parent = 0);
    virtual ~MPlayerConfig();

    virtual void setStatesByConfiguration(const Configuration& config);
    virtual void saveStatesToConfiguration();

    void focusMPlayerPathWidget() { mplayerPath_->setFocus(); }

public slots:
    void setDefaultSettings();

protected:
    virtual void createWidgets();

private slots:
    void execMPlayerPathDialog();

private:
    // Widgets
    QCheckBox *isOverlayEnabled_;
    QCheckBox *isDoubleBufferingEnabled_;
    QButtonGroup *framedropMethod_;
    QCheckBox *isAdditionalOptionsEnabled_;
    QLineEdit *additionalOptions_;
    QLineEdit *mplayerPath_;
    QCheckBox *isCacheEnabled_;
    QSpinBox *cacheSize_;
    QCheckBox *isPrefetchFileEnabled_;
};

class MPlayerFormatsConfig : public PlaybackBackendConfigWidget
{
public:
    MPlayerFormatsConfig(QWidget *parent = 0);
    virtual ~MPlayerFormatsConfig();

    virtual void setStatesByFormats(const QString& formats);
    virtual void formatsChanged(const QString& formats);
    virtual void saveStates();

private:
    QString formats_;
    QString orgFormats_;
    QLineEdit *mplayerArgumentsLineEdit_;
};

#endif // MPLAYERCONFIG_H_INCLUDED
